/*
 * utils.c
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "global.h"
#include "utils.h"


static FILE *logFile = NULL;


void error(char *fmt, ...) {
  FILE *log;
  va_list ap;

  log = (logFile != NULL ? logFile : stderr);
  va_start(ap, fmt);
  fprintf(log, "Error: ");
  vfprintf(log, fmt, ap);
  fprintf(log, "\n");
  va_end(ap);
  closeLog();
  if (log != stderr) {
    va_start(ap, fmt);
    fprintf(stderr, "Error: ");
    vfprintf(stderr, fmt, ap);
    fprintf(stderr, "\n");
    va_end(ap);
    fflush(stderr);
  }
  exit(1);
}


void writeLog(char *fmt, ...) {
  FILE *log;
  va_list ap;

  log = (logFile != NULL ? logFile : stderr);
  va_start(ap, fmt);
  vfprintf(log, fmt, ap);
  fprintf(log, "\n");
  va_end(ap);
  fflush(log);
  if (debug && log != stderr) {
    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    fprintf(stderr, "\n");
    va_end(ap);
    fflush(stderr);
  }
}


void openLog(char *logPath) {
  logFile = fopen(logPath, "w");
  if (logFile == NULL) {
    fprintf(stderr, "Warning: cannot open logfile '%s', ", logPath);
    fprintf(stderr, "using stderr instead!\n");
  }
}


void closeLog(void) {
  if (logFile != NULL) {
    fclose(logFile);
  }
}


char *protocolToString(int protocol) {
  switch (protocol) {
    case PROTOCOL_GMP:
      return "GMP";
    case PROTOCOL_SGMP:
      return "SGMP";
  }
  /* this should never happen */
  error("illegal protocol in protocolToString()");
  /* not reached */
  return NULL;
}


char *colorToString(Color color) {
  switch (color) {
    case NONE:
      return "none";
    case WHITE:
      return "white";
    case BLACK:
      return "black";
    case BOTH:
      return "both";
  }
  /* this should never happen */
  error("illegal color in colorToString()");
  /* not reached */
  return NULL;
}


char numToColumn(int n) {
  char result;

  result = 'A' + n;
  if (result >= 'I') {
    result++;
  }
  return result;
}


int columnToNum(char c) {
  int result;

  if (c >= 'a' && c <= 'z') {
    c -= 'a' - 'A';
  }
  result = c - 'A';
  if (result >= 8) {
    result--;
  }
  return result;
}


int numToRow(int n) {
  return n + 1;
}


int rowToNum(int r) {
  return r - 1;
}
